﻿using System;
using System.Collections.Generic;
using System.Text;
using ProcessSimulationInterfaces;

namespace ProcessSimulationSystem
{
    public class MotorProcess
    {
        private double K;       // Process constant
        private double Tau;     // Process time constant
        private double T;       // time per step
        private double ExpTTau; // intermediate calculation / constant
        private double K1Tau;   // intermediate calculation / constant
        private double Cxn;     // processoutpotlast C`n - last - motor position
        private double Cn;      // Processoutput - last - "motor speed"
        private double Mn;      // Processinput -  actual

        private SimAnalogValue processInput;
        private SimAnalogValue processOutput;
        private SimAnalogValue processxOutput;

        public MotorProcess(
                double K, double Tau, double T, 
                SimAnalogValue processInput,
                SimAnalogValue processOuput,
                SimAnalogValue processxOuput
                 )
        {
            this.processInput = processInput;
            this.processOutput = processOuput;
            this.processxOutput = processxOuput;

            this.K = K;
            this.Tau = Tau;
            this.T = T;

            this.ExpTTau = Math.Exp(-this.T / this.Tau);
            this.K1Tau = K * (1 - this.ExpTTau);
            this.Cxn = 0;
            this.Cn = 0;
            this.Mn = processInput.GetValue();
            this.processOutput.SetValue(this.Cn);
            this.processxOutput.SetValue(this.Cxn);
        }

        public void ResetProcessData()
        {
            this.Cxn = 0;
            this.Cn = 0;
            this.Mn = processInput.GetValue();
            this.processOutput.SetValue(this.Cn);
            this.processxOutput.SetValue(this.Cxn);
        }

        public void RunMotorProcess4OneTimeslice()
        {
            this.Mn = this.processInput.GetValue();

            this.Cxn = this.Cxn + (this.Cn * T / 2);    // old half of the time
            this.Cn = (this.ExpTTau * this.Cn) +
                           (this.K1Tau * this.Mn);
            this.Cxn = this.Cxn + (this.Cn * T / 2);    // new half of the time
                                                        // export values
            this.processOutput.SetValue(this.Cn);
            this.processxOutput.SetValue(this.Cxn);
        }
    }

}
