﻿using System;
using System.Collections.Generic;
using System.Text;
using ProcessSimulationInterfaces;

namespace ProcessSimulationSystem
{
    public class MotorProcessSystem : IMotorProcessSystem, IMotorProcessSystemSim
    {
        //public delegate void ProcessStateChangeEventType(IMotorProcessSystem motorProcesSystem);
        public event ProcessStateChangeEventType ProcessStateChangeEvent;
        public void SetProcessStateChangeEvent(ProcessStateChangeEventType e)
        {
            ProcessStateChangeEvent = e;
        }

        private volatile bool stop = false;

        public int SystemId { get; set; }

        public DigitalValue ProcessInputDigital { get; private set; }
        private SimAnalogValue processInputAnalog { get; set; }
        public SimAnalogValue ProcessOutputAnalog { get; private set; }
        public SimAnalogValue ProcessxOutputAnalog { get; private set; }
        private const double K = 1d;        // process measuring factor
        private const double Tau = 10d;     // process reaction
        private const double T = 0.1d;      // sampletime

        System.Threading.Thread processThread;

        private MotorProcess motorProcess;
        public MotorProcessSystem()
            : this(K, Tau, T)
        {
        }

        public MotorProcessSystem(double K, double Tau, double T)
        {
            ProcessInputDigital = new DigitalValue();
            processInputAnalog = new SimAnalogValue();
            ProcessOutputAnalog = new SimAnalogValue();
            ProcessxOutputAnalog = new SimAnalogValue();
            this.motorProcess = new MotorProcess(
                K, Tau, T, this.processInputAnalog, this.ProcessOutputAnalog, this.ProcessxOutputAnalog
                 );
        }

        public void ResetProcess()
        {
            ProcessInputDigital.SetValue(0);
            processInputAnalog.SetValue(0);
            motorProcess.ResetProcessData();
        }

        public void SetNewProcessInput(int value)
        {
            this.ProcessInputDigital.SetValue(value);
            this.processInputAnalog.SetValue(AD_DA_Convert.DigitalInput2AnalogPower(this.ProcessInputDigital.GetValue()));
            if (ProcessStateChangeEvent != null)
                ProcessStateChangeEvent(this);
        }

        public int GetProcessInput()
        {
            return this.ProcessInputDigital.GetValue();
        }

        public int GetProcessSpeed()
        {
            return AD_DA_Convert.AnalogSpeed2Digital(this.ProcessOutputAnalog.GetValue());
        }
        public int GetProcessPosition()
        {
            return AD_DA_Convert.AnalogPosition2Digital(this.ProcessxOutputAnalog.GetValue());
        }

        public void RunProcess()
        {
            while (!stop)
            //for (int i = 0; i < 10; ++i)
            {
                this.processInputAnalog.SetValue(AD_DA_Convert.DigitalInput2AnalogPower(this.ProcessInputDigital.GetValue()));

                this.motorProcess.RunMotorProcess4OneTimeslice();
                Console.WriteLine("i=" + this.processInputAnalog.GetValue().ToString("F8") + " "
                    + "o=" + this.ProcessOutputAnalog.GetValue().ToString("F8") + " "
                    + "ox=" + this.ProcessxOutputAnalog.GetValue().ToString("F8") + " "
                    );

                    if (ProcessStateChangeEvent != null)
                        ProcessStateChangeEvent(this);
                    System.Threading.Thread.Sleep(4);
            }
        }

        public void StartProcess()
        {
            if (processThread == null)
            {
                processThread = new System.Threading.Thread(this.RunProcess);
                processThread.IsBackground = true;
                processThread.Start();
            }
        }
    }
}
